-- ===========================================================================
-- Prospector 3.0 - Plugin for Vendetta Online (all platforms)
-- ===========================================================================
-- Originally created by PaKettle, 13 August 2016 to 11 August 2020
-- This version largely rewritten by Snib#9290 in June 2022

-- All data and settings are stored in .../settings/<Player>/system91*.txt

-- Fully configurable via GUI

-- User commands:
--  /pr /prospector             - show GUI
--  /rs /roidscan               - toggle automatic asteroid scanner
--
--  /prForceDatabaseUpgrade     - force a database upgrade
--
--  /prImportFromTargetless     - import new ore data from Targetless
--  /prExportToTargetless       - export ore data to Targetless
-- ===========================================================================

declare("PR", {})
PR = {
    Version = "3.0.9~Snib",
    Sector = 0, -- current sector id
    LoadedSector = 0, -- id of currently loaded sector
    PluginTitle = "",
    PluginReady = false, -- true after PR.Init() ran
    SystemNotesPrefix = 91000000, -- prefix for notes stored by this plugin
    VerbosityLevels = { "Quiet", "Normal", "Noisy" },
    GUI = {
        Color = {}, -- presets for iup fgcolor/bgcolor
        EditableValues = { "StartupDelay", "AutoScanFrequency", "MinXPV" },
        EditableValuesUnit = { "ms", "ms", "XPV" },
        SectorRoidList = {}, -- caches asteroid list while GUI is open
        SystemData = {}, -- caches data for all loaded systems while GUI is open
        CurrentSectorShowing = false, -- whether the GUI is open showing the current sector
        DisplayedSector = 0, -- id of sector shown by sector view
        ListedItems = {}, -- ids of displayed asteroids
        MaxDisplayRows = 0, -- length if displayed list
        View = "Sector", -- currently active view
        SectorOres = {}, -- list of ores in currently displayed sector
        OreSort = 0, -- id of the ore in SectorOres that ByOre is sorting by
        HasPopup = true
    },
    AutoScanActive = false,
    AutoScanTimer = Timer(),
    SingleSearchTarget = 0, -- single search mode if this is set to an asteroid id
    SingleSearchTimer = Timer(),
    LeviDelayCount = 0,
    LeviTimer = Timer(),
    SpamHold = 0,
    SpamTimer = Timer(),
    LastDisplayedRoidId = 0, -- keep track of last displayed roid so we don't repeat it
    RoidNode = 0, -- radar node for asteroids in the current sector
    MiningBeamEquipped = false,
    ScannerEquipped = false,
    ScanRange = 0,
    MassTarget = 0, -- mass test target asteroid id
    MassTestActive = false,
    MassScanCount = 0, -- mass test scan count
    MassScanMax = 0, -- max mass test scan count, measurement when reached
    LastTemp = 0, -- last measured asteroid temperature
    BeamAvgHeat = 0, -- average heat generated by all equipped mining beams
    SaveSectorId = 0, -- id of sector to be saved
    SaveSectorTimer = Timer(),
    SystemUpdateSectorId = 0, -- system data update pending for this sector
    SystemUpdateTimer = Timer(),
    Config = {}, -- user configuration
    Data = {}, -- data for currently loaded sector
    Color = {}, -- text color presets
    Ore = {},
    ShipsWithMineralScanner = {},
    SystemOwnership = {},
    IsBotwatcherInstalled = false,
    IsTargetlessInstalled = false,
}
PR.PluginTitle = "Prospector v" .. PR.Version
-- checking if variables exist is complicated in lua
pcall(function() return targetless.api end)
PR.GUI.HasPopup = pcall(function() return iup.Popup end)

dofile("static_data.lua")

PR.Config = {
    Version = PR.Version, -- Version string determines potential upgrade process
    AutoScanFrequency = 1000, -- frequency in ms for auto scanner to retarget
    StartupDelay = 5000, -- delay before registering most event handlers
    MinXPV = 1, -- asteroids with XPV below this will be shown as "junk"
    MaxIdDigits = 2, -- minimum digits for roid id leading 0 formatting
    SectorSort = "ById", -- GUI sector sorting
    UniverseSort = "BySystem", -- GUI Universe sorting
    Search = { Ore = PR.Ore[1].shortname, Area = "Gray", System = 33 }, -- search settings
    COLOR = true, -- status message output in color or white only
    VerbosityLevel = 3, -- controls which messages are printed and how often
    PrintToHUD = true -- controls whether some messages get printed on HUD
}

dofile("utils.lua")
dofile("equipment.lua")
dofile("reminder.lua")
dofile("roid.lua")
dofile("roid_display.lua")
dofile("scanners.lua")
dofile("scan_event.lua")
dofile("sector.lua")
dofile("system.lua")
dofile("targetless.lua")
dofile("upgrade.lua")
dofile("GUI/main.lua")
-- load debug code, if present
local loadDebugChunk = loadfile("debug.lua")
if loadDebugChunk then
    loadDebugChunk()
    loadDebugChunk = nil
end

function PR.Init()
    UnregisterEvent(PR.Init, "ENTERED_STATION")
    UnregisterEvent(PR.Init, "SHIP_SPAWNED")

    -- config needs to be loaded first
    local config = unspickle(LoadSystemNotes(PR.SystemNotesPrefix)) or {}
    for key, val in pairs(config) do
        PR.Config[key] = val
    end

    -- do any plugin version upgrade before loading any further data
    if PR.Version ~= PR.Config.Version then
        PR.UpgradeVersion()
    end

    -- load ship and sector data
    PR.CheckEquipment()
    PR.SectorChanged()

    -- Workaround for "invalid key to 'next'" crash when Targetless is also
    -- installed, which seems to be a bug on the game's side, iterating over
    -- the list of handlers while adding and removing them
    PR.SingleSearchTimer:SetTimeout(PR.Config.StartupDelay, PR.RegisterEvents)
end

function PR.RegisterEvents()
    RegisterEvent(PR.TargetScanned, "TARGET_SCANNED")
    RegisterEvent(PR.FreshStart, "ENTERING_STATION")
    RegisterEvent(PR.SectorChanged, "SECTOR_CHANGED")
    RegisterEvent(PR.CheckEquipment, "SHIP_UPDATED")
    RegisterEvent(PR.CheckEquipment, "SHIP_SPAWNED")
    RegisterEvent(PR.CheckEquipment, "SHIP_CHANGED")
    RegisterEvent(PR.SaveNow, "PLAYER_LOGGED_OUT")
    RegisterEvent(PR.SaveNow, "UNLOAD_INTERFACE")

    PR.Print(PR.PluginTitle .. " ready", true)
    PR.PluginReady = true
end

function PR.CheckOtherPlugins()
    PR.IsTargetlessInstalled = pcall(function() return targetless.api.radarlock end) and
        pcall(function() return targetless.var.lock end)
    PR.IsBotwatcherInstalled = pcall(function() return Botwatcher.GetBotsSightedList end)
end

RegisterEvent(PR.CheckOtherPlugins, "PLUGINS_LOADED")

-- init as soon as we can check equipment
RegisterEvent(PR.Init, "ENTERED_STATION")
RegisterEvent(PR.Init, "SHIP_SPAWNED")

-- ==============
-- SLASH COMMANDS
-- ==============
RegisterUserCommand("roidscan", PR.AutoScanToggle)
RegisterUserCommand("rs", PR.AutoScanToggle)

RegisterUserCommand("prospector", PR.GUI.ShowGUI)
RegisterUserCommand("pr", PR.GUI.ShowGUI)
RegisterUserCommand("ore", PR.GUI.ShowGUI) -- deprecated

RegisterUserCommand("prForceDatabaseUpgrade", PR.ForceUpgradeVersion)

RegisterUserCommand("prImportFromTargetless", PR.ImportFromTargetless)
RegisterUserCommand("prExportToTargetless", PR.ExportToTargetless)
